@@ -8,7 +8,7 @@ no warnings 'recursion';  # trees can be quite deep
 
 package XML::Compile::Translate;
 use vars '$VERSION';
-$VERSION = '1.56';
+$VERSION = '1.57';
 
 
 # Errors are either in _class 'usage': called with request
@@ -448,8 +448,9 @@ sub simpleRestriction($$)
     +{ st => $do };
 }
 
-my %facets_early = map +($_ => 1), qw/whiteSpace pattern enumeration/;
-#my %facets_late = map +($_ => 1), qw/totalDigits maxScale minScale
+# Early=lexical space, Late=value space
+my %facets_early = map +($_ => 1), qw/whiteSpace pattern/;
+#my %facets_late = map +($_ => 1), qw/totalDigits maxScale minScale enumeration
 #   maxInclusive maxExclusive minInclusive minExclusive fractionDigits
 #   length minLength maxLength/;
 
@@ -527,7 +528,7 @@ sub applySimpleFacets($$$$)
 
       $is_list
     ? $self->makeFacetsList($where, $st, \%facets_info, \@early, \@late)
-    : $self->makeFacets($where, $st, \%facets_info, @early, @late);
+    : $self->makeFacets($where, $st, \%facets_info, \@early, \@late);
 }
 
 sub element($;$)
